import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.event.*;

/*
 * FinchDisplay class
 * 
 * <P>
 * Create a frame that contains components for displaying Finch sensor
 * values and allows update of the nose color.
 * 
 * 
 */

abstract public class FinchDisplayAbstract extends JFrame {

	// References to the text labels that will display sensor values
	protected JLabel lightLeft;
	protected JLabel lightRight;
	protected JLabel accelX;
	protected JLabel accelY;
	protected JLabel accelZ;
	protected JLabel obstacleLeft;
	protected JLabel obstacleRight;
	protected JLabel temperature;
	
	// RGB sliders
	protected final JSlider slider1 = new JSlider(0, 255, 0);
	protected final JSlider slider2 = new JSlider(0, 255, 0);
	protected final JSlider slider3 = new JSlider(0, 255, 0);
	
	// RGB values
	protected int LED[];
	
	// Set to true if the window is open; false means that the window is closing
	protected boolean openFlag;

	// Components for FinchDisplay
	// TODO: configure these components in your implementation of FinchDisplay
	
	// Menu
	protected JMenuBar menuBar = new JMenuBar();
	protected JMenu fileMenu = new JMenu("Menu");
	protected JMenuItem jmiExit = new JMenuItem("Exit");
	protected JMenuItem jmiColorChooser = new JMenuItem("Set Nose Color");
	
	// Tool bar
	protected JButton jbtColorChooser = new JButton(new ImageIcon(getClass().getResource("colorchooser.gif")));
	protected JButton jbtExit = new JButton(new ImageIcon(getClass().getResource("icon-exit.gif")));
	
	
	/**
	 * Constructor of the Finch display frame.  Creates all of the
	 * frame contents.
	 *  
	 */
	
	public FinchDisplayAbstract()
	{	
		// Initialize the RGB LED array
		this.LED = new int[3];
		
		// Indicate that the window is open (or will soon be)
		this.openFlag = true;
		
		// Set up a grid that will have 1 column (and an arbitrary number of rows)
		this.setLayout(new GridLayout(0, 1, 10, 10));
		
		/////////////////////////////////////////////////////////
		// Light panel
		JPanel p1 = new JPanel(new GridLayout(0, 2, 10, 10));
		// Border for this pair of values
		p1.setBorder(new TitledBorder("Light Sensors"));

		// Value to be displayed.  Note, we save a reference to this object
		//   so that we can update it later as data arrive.
		lightLeft = new JLabel("none");
		p1.add(lightLeft);
		
		// Value to be displayed
		lightRight = new JLabel("none");
		p1.add(lightRight);
		
		// Add this panel to the frame
		this.add(p1);
		
		/////////////////////////////////////////////////
		// Accelerometer panel
		JPanel Paccel = new JPanel(new GridLayout(0, 3, 10, 10));
		// Border for this triple of values
		Paccel.setBorder(new TitledBorder("Accelerometers"));
		
		accelX = new JLabel("none");
		Paccel.add(accelX);
		
		accelY= new JLabel("none");
		Paccel.add(accelY);
		
		accelZ = new JLabel("none");
		Paccel.add(accelZ);
		
		// Add this panel to the frame
		this.add(Paccel);
		
		/////////////////////////////////////////////////
		// Obstacle panel
		JPanel Pobstacle = new JPanel(new GridLayout(0, 2, 10, 10));
		
		// Border for this pair of values
		Pobstacle.setBorder(new TitledBorder("Obstacle"));

		obstacleLeft = new JLabel("none");
		Pobstacle.add(obstacleLeft);
		
		obstacleRight = new JLabel("none");
		Pobstacle.add(obstacleRight);
		
		// Add this panel to the frame
		this.add(Pobstacle);
		
		/////////////////////////////////////////////////
		// Temperature panel
		JPanel Ptemperature = new JPanel(new GridLayout(0, 1, 10, 10));
		
		// Border for this pair of values
		Ptemperature.setBorder(new TitledBorder("Temperature"));
		temperature = new JLabel("none");
		Ptemperature.add(temperature);
		
		// Add this panel to the frame
		this.add(Ptemperature);
		
		/////////////////////////
		JPanel Pslider = new JPanel(new GridLayout(0, 1, 10, 10));
		
		// Slider for Red LED
		slider1.setBackground(new Color(255,150,150));
		
		// Set up a listener: stateChanged event is called when 
		//  the slider is moved 
		// This is an 'anonymous class' definition: it is a class
		//  that extends ChangeListener but doesn't have a name itself.
		//  We are doing this because we only need one instance of the class
		//  and it only defines implementations for abstracted methods
		//  (we will get into these details later).
		slider1.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e){
				// Note: LED and slider1 are instance variables of FinchDisplay &
				//  are within the scope of this new class
				LED[0] = slider1.getValue();
			}
		});
		Pslider.add(slider1);
		
		// Slider for Green LED
		slider2.setBackground(new Color(150,255,150));
		
		// Set up a listener: stateChanged event is called when 
		//  the slider is moved.
		slider2.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e){
				LED[1] = slider2.getValue();
			}
		});
		Pslider.add(slider2);
		
		// Slider for Blue LED
		slider3.setBackground(new Color(150,150,255));
		
		// Set up a listener: stateChanged event is called when 
		//  the slider is moved.
		slider3.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e){
				LED[2] = slider3.getValue();
			}
		});
		
		Pslider.add(slider3);
		
		
		// Finish off Slider panel configuration: set the name and attach it 
		//  to the frame
		
		Pslider.setBorder(new TitledBorder("RGB"));
		this.add(Pslider);
		
		//////////////////////////
		// React to window events (we will get into these details 
		//  later).  This is also an anonymous class definition.
		this.addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent event) {
				// Respond to the close event (a click of the 'X' button
				//  in the upper-left corner of the frame).
				closeFinch();
				System.out.println("closing...");
			}
		});
		
	}
	
	/**
	 *  Method to be called in response to window being closed.
	 *   
	 *  <P>
	 *  Postconditions:
	 * <OL> 
	 * <LI>  The openFlag is set to false to indicate that the driver
	 * program should shut down
	 * </OL>
	 */
	protected void closeFinch()
	{
		this.openFlag = false;
	}
	
	/**
	 * Returns the open flag, which indicates whether the window is open
	 * (true) or closed/closing (false)
	 * 
	 * @return The state of the "open" flag
	 */
	public boolean getOpenFlag()
	{
		return(this.openFlag);
	}
	
	/**
	 * Change the light sensor values that are displayed
	 * 
	 * @param left	The value of the left light sensor
	 * @param right The value of the right light sensor
	 * 
	 */
	public void setLight(Integer left, Integer right)
	{
		lightLeft.setText(left.toString());
		lightRight.setText(right.toString());
	}
	
	/**
	 * Change the acceleration values that are displayed
	 * 
	 * @param x	Acceleration along the X axis (g)
	 * @param y Y axis
	 * @param z Z axis
	 */
	public void setAccel(Double x, Double y, Double z)
	{
		accelX.setText(x.toString());
		accelY.setText(y.toString());
		accelZ.setText(z.toString());
	}
	
	/**
	 * Change the obstacle values that are displayed
	 * 
	 * @param left Left obstacle sensor (boolean)
	 * @param right Right sensor
	 */
	public void setObstacle(Boolean left, Boolean right)
	{
		obstacleLeft.setText(left.toString());
		obstacleRight.setText(right.toString());
		
	}
	
	/**
	 * Change the temperature value that is displayed
	 * 
	 * @param temp Temperature (in F)
	 */
	public void setTemperature(Double temp)
	{
		temperature.setText(temp.toString());
	}
	
	/**
	 * Return the RGB values specified by the sliders.
	 * 
	 * @return The RGB array that contains the current desired values of the LED.
	 */
	public int[] getLED(){
		return(LED);
	}
	
	/**
	 * 
	 * <P>
	 * 
	 *  Open up a color chooser dialog box with a default color equal to 
	 *  the current state of the "LED" property.  If a new color is selected,
	 *  set the LED property AND the RGB slider bars to be equal to this new
	 *  color
	 *  
	 *  <P>
	 *  
	 *  Postconditions:
	 *  <OL>
	 *  <LI> slider1, slider2, slider3 have been changed to match the selected color
	 *  <LI> this.LED has been altered to also match the selected color
	 *  </OL>
	 *
	 */
	abstract protected void colorChooser();
	
	
	/**
	 * 
	 * <P>
	 * 
	 *  Open a ConfirmDialog box that asks whether the user really wants to
	 *  quit the program.  If "Yes" is selected, then the program is shut
	 *  down by calling closeFinch()
	 *  
	 *  Postconditions:
	 *  <OL>
	 *  <LI> this.closeFinch() will have been called if indicated by the user.
	 *  </OL>
	 *
	 *<P>
	 *
	 *  Notes:
	 *  <OL>
	 *  <LI> JOptionPane.showConfirmDialog will return a "0" if "Yes" is selected.
	 *  <LI> The dialog box should only present two buttons: "Yes" and "No"
	 *  </OL>
	 *  
	 */
	abstract protected void exitChooser();
	
	
}
