import javax.swing.JFrame;
import finch.Finch;


/*
 * lab5Driver: main driver program
 * 
 * <P>
 * 
 * This driver program opens a window for displaying the sensor values
 * that are read from the Finch.
 * In addition, the user can specify using slider bars the brightness of 
 * each of the LEDs in the nose of the Finch.
 * 
 * <P>
 * 
 * @author Andrew H. Fagg
 * 
 */

public class lab7Driver {
	
	/**
	 * Sleep for a specified amount of time.
	 * 
	 * @param i Number of milliseconds to sleep
	 */
	
	static public void mySleep(int i)
	{
		try{
			Thread.sleep(i);
		}catch(Exception e)
		{
			System.out.println(e);
		}
	}
	
	/**
	 * Sensor loop: continually read the sensor values from the Finch and
	 *  display them.   In addition, set the Finch LED state to that indicated
	 *  by the slider bars.
	 *   
	 * @param display The frame being displayed in the window
	 * @param myFinch A reference to the Finch object
	 */
	
	static public void sensorLoop(FinchDisplayAbstract display, Finch myFinch)
	{
		// Sensor values from the Finch
		int light[] = new int[2];
		double acceleration[] = new double[3];
		double temperature = 100.0;
		boolean obstacle[] = new boolean[2];
		
		// LED values specified by the user (with the slider bars)
		int LED[];
		
		// Loop until the display indicates that it should be closed
		while(display.getOpenFlag()) {
			// Get the current sensor values
			if(myFinch != null){
				light = myFinch.getLightSensors();
				acceleration = myFinch.getAccelerations();
				temperature = myFinch.getTemperature();
				obstacle = myFinch.getObstacleSensors();
			}
			// Display these sensor values in the window
			display.setLight(light[0], light[1]);
			display.setAccel(acceleration[0], acceleration[1], acceleration[2]);
			display.setTemperature(temperature);
			display.setObstacle(obstacle[0], obstacle[1]);
			
			// Get the LED values specified by the slider bars
			LED = display.getLED();
			
			// Set the Finch nose to this light level
			if(myFinch != null){
				myFinch.setLED(LED[0], LED[1], LED[2]);
			}
			// Wait for a small period of time.
			mySleep(10);
		}
	}
	
	/**
	 * Main method
	 * <P>
	 * <OL>
	 * <LI> Create the frame to be displayed
	 * <LI> Configure the window to display the frame
	 * <LI> Enter the sensor loop that will continuously update the display
	 * </OL>
	 * 
	 * @param args Arguments are ignored
	 */
	public static void main(String args[]){
		// Connect to the Finch
		Finch myFinch = null;
		//myFinch = new Finch();
		
		// Create the display frame
		FinchDisplay display = new FinchDisplay();
		
		// Title
		display.setTitle("Finch Data");
		
		// Size of window
		display.setSize(400,600);
		
		// Set window location to middle of screen
		display.setLocationRelativeTo(null);
		
		// When window is closed: exit the program
		display.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		// Display the window
		display.setVisible(true);
		
		// Display sensor values live
		sensorLoop(display, myFinch);
		System.out.println("Exiting....");
		
		// Clean up
		if(myFinch != null){
			myFinch.stopWheels();
			myFinch.quit();
		}
		// Force exit
		System.exit(0);
	}
}
